@@ -56,16 +56,16 @@ static Client *pending_focus = NULL;
 static Client *last_ungrab   = NULL;
 
 static ClientPair
-clientGetTopMostFocusable (ScreenData *md, int layer, Client * exclude)
+clientGetTopMostFocusable (ScreenInfo *screen_info, int layer, Client * exclude)
 {
     ClientPair top_client;
-    Client *c;
-    GList *index;
+    Client *c = NULL;
+    GList *index = NULL;
 
     TRACE ("entering clientGetTopMostFocusable");
 
     top_client.prefered = top_client.highest = NULL;
-    for (index = windows_stack; index; index = g_list_next (index))
+    for (index = screen_info->windows_stack; index; index = g_list_next (index))
     {
         c = (Client *) index->data;
         TRACE ("*** stack window \"%s\" (0x%lx), layer %i", c->name,
@@ -81,7 +81,7 @@ clientGetTopMostFocusable (ScreenData *md, int layer, Client * exclude)
         {
             if ((c->win_layer <= layer) && FLAG_TEST (c->flags, CLIENT_FLAG_VISIBLE))
             {
-                if (clientSelectMask (c, 0))
+                if (clientSelectMask (c, 0, WINDOW_NORMAL | WINDOW_DIALOG | WINDOW_MODAL_DIALOG))
                 {
                     top_client.prefered = c;
                 }
@@ -98,25 +98,26 @@ clientGetTopMostFocusable (ScreenData *md, int layer, Client * exclude)
 }
 
 void
-clientFocusTop (ScreenData *md, int layer)
+clientFocusTop (ScreenInfo *screen_info, int layer)
 {
     ClientPair top_client;
 
-    top_client = clientGetTopMostFocusable (md, layer, NULL);
+    top_client = clientGetTopMostFocusable (screen_info, layer, NULL);
     if (top_client.prefered)
     {
-        clientSetFocus (md, top_client.prefered, GDK_CURRENT_TIME, NO_FOCUS_FLAG);
+        clientSetFocus (screen_info, top_client.prefered, GDK_CURRENT_TIME, NO_FOCUS_FLAG);
     }
     else
     {
-        clientSetFocus (md, top_client.highest, GDK_CURRENT_TIME, NO_FOCUS_FLAG);
+        clientSetFocus (screen_info, top_client.highest, GDK_CURRENT_TIME, NO_FOCUS_FLAG);
     }
 }
 
 void
 clientFocusNew(Client * c)
 {
-    gboolean give_focus = params.focus_new;
+    ScreenInfo *screen_info = NULL;
+    gboolean give_focus;
 
     g_return_if_fail (c != NULL);
     
@@ -124,7 +125,8 @@ clientFocusNew(Client * c)
     {
         return;
     }
-
+    screen_info = c->screen_info;
+    give_focus = screen_info->params->focus_new;
 #if 0    
     /*  Try to avoid focus stealing */
     if (client_focus)
@@ -143,7 +145,14 @@ clientFocusNew(Client * c)
     
     if (give_focus || FLAG_TEST(c->flags, CLIENT_FLAG_STATE_MODAL))
     {
-        clientSetFocus (c->md, c, GDK_CURRENT_TIME, FOCUS_IGNORE_MODAL);
+        if (client_focus)
+        {
+            if (clientAdjustFullscreenLayer (client_focus, FALSE))
+            {
+                clientRaise (c);
+            }
+        }
+        clientSetFocus (c->screen_info, c, GDK_CURRENT_TIME, FOCUS_IGNORE_MODAL);
         clientPassGrabButton1 (c);
     }
     else
@@ -153,55 +162,56 @@ clientFocusNew(Client * c)
 }
 
 gboolean
-clientSelectMask (Client * c, int mask)
+clientSelectMask (Client * c, int mask, int type)
 {
-    gboolean okay = TRUE;
-
     TRACE ("entering clientSelectMask");
+
+    g_return_val_if_fail (c != NULL, FALSE);
+
     if ((!clientAcceptFocus (c)) && !(mask & INCLUDE_SKIP_FOCUS))
     {
-        okay = FALSE;
+        return FALSE;
     }
     if (FLAG_TEST (c->flags, CLIENT_FLAG_HIDDEN) && !(mask & INCLUDE_HIDDEN))
     {
-        okay = FALSE;
+        return FALSE;
     }
     if (FLAG_TEST (c->flags, CLIENT_FLAG_SKIP_PAGER)
         && !(mask & INCLUDE_SKIP_PAGER))
     {
-        okay = FALSE;
+        return FALSE;
     }
     if (FLAG_TEST (c->flags, CLIENT_FLAG_SKIP_TASKBAR)
         && !(mask & INCLUDE_SKIP_TASKBAR))
     {
-        okay = FALSE;
+        return FALSE;
     }
-    if ((c->win_workspace != c->md->current_ws) && !(mask & INCLUDE_ALL_WORKSPACES))
+    if ((c->win_workspace != c->screen_info->current_ws) && !(mask & INCLUDE_ALL_WORKSPACES))
     {
-        okay = FALSE;
+        return FALSE;
     }
-
-    return okay;
+    if (c->type & type)
+    {
+        return TRUE;
+    }
+    return FALSE;
 }
 
 Client *
 clientGetNext (Client * c, int mask)
 {
-    Client *c2;
+    Client *c2 = NULL;
     unsigned int i;
 
     TRACE ("entering clientGetNext");
 
     if (c)
     {
-        for (c2 = c->next, i = 0; (c2) && (i < client_count);
+        ScreenInfo *screen_info = c->screen_info;
+        for (c2 = c->next, i = 0; (c2) && (i < screen_info->client_count - 1);
             c2 = c2->next, i++)
         {
-            if (c2->type & (WINDOW_SPLASHSCREEN | WINDOW_DOCK | WINDOW_DESKTOP))
-            {
-                continue;
-            }
-            if (clientSelectMask (c2, mask))
+            if (clientSelectMask (c2, mask, WINDOW_NORMAL | WINDOW_DIALOG | WINDOW_MODAL_DIALOG))
             {
                 return c2;
             }
@@ -213,21 +223,18 @@ clientGetNext (Client * c, int mask)
 Client *
 clientGetPrevious (Client * c, int mask)
 {
-    Client *c2;
+    Client *c2 = NULL;
     unsigned int i;
 
     TRACE ("entering clientGetPrevious");
 
     if (c)
     {
-        for (c2 = c->prev, i = 0; (c2) && (i < client_count);
+        ScreenInfo *screen_info = c->screen_info;
+        for (c2 = c->prev, i = 0; (c2) && (i < screen_info->client_count);
             c2 = c2->prev, i++)
         {
-            if (c2->type & (WINDOW_SPLASHSCREEN | WINDOW_DOCK | WINDOW_DESKTOP))
-            {
-                continue;
-            }
-            if (clientSelectMask (c2, mask))
+            if (clientSelectMask (c2, mask, WINDOW_NORMAL | WINDOW_DIALOG | WINDOW_MODAL_DIALOG))
             {
                 return c2;
             }
@@ -237,12 +244,12 @@ clientGetPrevious (Client * c, int mask)
 }
 
 void
-clientPassFocus (ScreenData *md, Client * c)
+clientPassFocus (ScreenInfo *screen_info, Client * c)
 {
     Client *new_focus = NULL;
     Client *current_focus = client_focus;
     ClientPair top_most;
-    Client *c2;
+    Client *c2 = NULL;
     Window dr, window;
     int rx, ry, wx, wy;
     unsigned int mask;
@@ -260,35 +267,42 @@ clientPassFocus (ScreenData *md, Client * c)
         return;
     }
 
-    top_most = clientGetTopMostFocusable (md, look_in_layer, c);
-    if (params.click_to_focus)
+    top_most = clientGetTopMostFocusable (screen_info, look_in_layer, c);
+    if (screen_info->params->click_to_focus)
     {
-        if ((c) && clientIsModal (c))
+        if (c)
         {
-            /* If the window is a modal, send focus back to its parent window
-               Modals are transients, and we aren"t interested in modal
-               for group, so it safe to use clientGetTransient because 
-               it's really what we want...
-             */
-
-            c2 = clientGetTransient (c);
-            if (c2 && FLAG_TEST(c2->flags, CLIENT_FLAG_VISIBLE))
+            if (clientIsModal (c))
             {
-                new_focus = c2;
-                /* Usability: raise the parent, to grab user's attention */
-                clientRaise (c2);
+                /* If the window is a modal, send focus back to its parent window
+                   Modals are transients, and we aren"t interested in modal
+                   for group, so it safe to use clientGetTransient because 
+                   it's really what we want...
+                 */
+
+                c2 = clientGetTransient (c);
+                if (c2 && FLAG_TEST(c2->flags, CLIENT_FLAG_VISIBLE))
+                {
+                    new_focus = c2;
+                    /* Usability: raise the parent, to grab user's attention */
+                    clientRaise (c2);
+                }
+            }
+            else
+            {
+                new_focus = clientGetNext (c, 0);
             }
         }
     }
-    else if (XQueryPointer (md->dpy, md->xroot, &dr, &window, &rx, &ry, &wx, &wy, &mask))
+    else if (XQueryPointer (myScreenGetXDisplay (screen_info), screen_info->xroot, &dr, &window, &rx, &ry, &wx, &wy, &mask))
     {
-        new_focus = clientAtPosition (rx, ry, c);
+        new_focus = clientAtPosition (screen_info, rx, ry, c);
     }
     if (!new_focus)
     {
         new_focus = top_most.prefered ? top_most.prefered : top_most.highest;
     }
-    clientSetFocus (md, new_focus, GDK_CURRENT_TIME, FOCUS_IGNORE_MODAL | FOCUS_FORCE);
+    clientSetFocus (screen_info, new_focus, GDK_CURRENT_TIME, FOCUS_IGNORE_MODAL | FOCUS_FORCE);
     if (new_focus == top_most.highest)
     {
         clientPassGrabButton1 (new_focus);
@@ -327,24 +341,27 @@ clientAcceptFocus (Client * c)
 void
 clientSortRing(Client *c)
 {
+    ScreenInfo *screen_info = NULL;
+    
     g_return_if_fail (c != NULL);
 
     TRACE ("Sorting...");
-    if (client_count > 2 && c != clients)
+    screen_info = c->screen_info;
+    if (screen_info->client_count > 2 && c != screen_info->clients)
     {
         c->prev->next = c->next;
         c->next->prev = c->prev;
 
-        c->prev = clients->prev;
-        c->next = clients;
-        clients->prev->next = c;
-        clients->prev = c;
+        c->prev = screen_info->clients->prev;
+        c->next = screen_info->clients;
+        screen_info->clients->prev->next = c;
+        screen_info->clients->prev = c;
     }
-    clients = c;
+    screen_info->clients = c;
 }
 
 void
-clientUpdateFocus (ScreenData *md, Client * c, unsigned short flags)
+clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
 {
     Client *c2 = ((client_focus != c) ? client_focus : NULL);
     unsigned long data[2];
@@ -357,10 +374,9 @@ clientUpdateFocus (ScreenData *md, Client * c, unsigned short flags)
         TRACE ("SKIP_FOCUS set for client \"%s\" (0x%lx)", c->name, c->window);
         return;
     }
-    if ((c == client_focus) && !(flags & FOCUS_FORCE))
+    if ((c) && (c == client_focus) && !(flags & FOCUS_FORCE))
     {
-        TRACE ("client \"%s\" (0x%lx) is already focused, ignoring request",
-            c->name, c->window);
+        TRACE ("client \"%s\" (0x%lx) is already focused, ignoring request", c->name, c->window);
         return;
     }
     client_focus = c;
@@ -372,10 +388,7 @@ clientUpdateFocus (ScreenData *md, Client * c, unsigned short flags)
             clientSortRing(c);
         }
         data[0] = c->window;
-        if ((c->legacy_fullscreen) || FLAG_TEST(c->flags, CLIENT_FLAG_FULLSCREEN))
-        {
-            clientSetLayer (c, WIN_LAYER_ABOVE_DOCK);
-        }
+        clientAdjustFullscreenLayer (c, TRUE);
         frameDraw (c, FALSE, FALSE);
     }
     else
@@ -384,40 +397,23 @@ clientUpdateFocus (ScreenData *md, Client * c, unsigned short flags)
     }
     if (c2)
     {
-        TRACE ("redrawing previous focus client \"%s\" (0x%lx)", c2->name,
-            c2->window);
-        /* Requires a bit of explanation here... Legacy apps automatically
-           switch to above layer when receiving focus, and return to
-           normal layer when loosing focus.
-           The following "logic" is in charge of that behaviour.
-         */
-        if ((c2->legacy_fullscreen) || FLAG_TEST(c2->flags, CLIENT_FLAG_FULLSCREEN))
+        if (c)
         {
-            if (FLAG_TEST(c2->flags, CLIENT_FLAG_FULLSCREEN))
-            {
-                clientSetLayer (c2, c2->fullscreen_old_layer);
-            }
-            else
-            {
-                clientSetLayer (c2, WIN_LAYER_NORMAL);
-            }
-            if (c)
-            {
-                clientRaise(c);
-                clientPassGrabButton1 (c);
-            }
+            clientAdjustFullscreenLayer (c2, FALSE);
+            clientRaise (c);
+            clientPassGrabButton1 (c);
         }
         frameDraw (c2, FALSE, FALSE);
     }
     data[1] = None;
-    XChangeProperty (md->dpy, md->xroot, net_active_window, XA_WINDOW, 32,
+    XChangeProperty (myScreenGetXDisplay (screen_info), screen_info->xroot, net_active_window, XA_WINDOW, 32,
         PropModeReplace, (unsigned char *) data, 2);
 }
 
 void
-clientSetFocus (ScreenData *md, Client * c, Time timestamp, unsigned short flags)
+clientSetFocus (ScreenInfo *screen_info, Client * c, Time timestamp, unsigned short flags)
 {
-    Client *c2;
+    Client *c2 = NULL;
 
     TRACE ("entering clientSetFocus");
     
@@ -448,13 +444,12 @@ clientSetFocus (ScreenData *md, Client * c, Time timestamp, unsigned short flags
         if (FLAG_TEST (c->wm_flags, WM_FLAG_INPUT))
         {
             pending_focus = c;
-            XSetInputFocus (md->dpy, c->window, RevertToPointerRoot, timestamp);
+            XSetInputFocus (myScreenGetXDisplay (screen_info), c->window, RevertToPointerRoot, timestamp);
         }
         if (FLAG_TEST(c->wm_flags, WM_FLAG_TAKEFOCUS))
         {
-            sendClientMessage (c->md, c->window, wm_protocols, wm_takefocus, timestamp);
+            sendClientMessage (c->screen_info, c->window, wm_protocols, wm_takefocus, timestamp);
         }
-        XFlush (md->dpy);
     }
     else
     {
@@ -467,13 +462,12 @@ clientSetFocus (ScreenData *md, Client * c, Time timestamp, unsigned short flags
         if (c2)
         {
             frameDraw (c2, FALSE, FALSE);
-            XChangeProperty (c2->md->dpy, c2->md->xroot, net_active_window, XA_WINDOW, 32,
+            XChangeProperty (clientGetXDisplay (c2), c2->screen_info->xroot, net_active_window, XA_WINDOW, 32,
                 PropModeReplace, (unsigned char *) data, 2);
         }
-        XChangeProperty (md->dpy, md->xroot, net_active_window, XA_WINDOW, 32,
+        XChangeProperty (myScreenGetXDisplay (screen_info), screen_info->xroot, net_active_window, XA_WINDOW, 32,
             PropModeReplace, (unsigned char *) data, 2);
-        XSetInputFocus (md->dpy, md->gnome_win, RevertToPointerRoot, GDK_CURRENT_TIME);
-        XFlush (md->dpy);
+        XSetInputFocus (myScreenGetXDisplay (screen_info), screen_info->gnome_win, RevertToPointerRoot, GDK_CURRENT_TIME);
     }
 }
 
@@ -496,7 +490,7 @@ clientGrabButton1 (Client * c)
     TRACE ("entering clientGrabButton1");
     TRACE ("grabbing buttons for client \"%s\" (0x%lx)", c->name, c->window);
     
-    grabButton(c->md->dpy, Button1, 0, c->window);
+    grabButton(clientGetXDisplay (c), Button1, 0, c->window);
 }
 
 void
@@ -506,7 +500,7 @@ clientUngrabButton1 (Client * c)
     TRACE ("entering clientUngrabButton1");
     TRACE ("ungrabing buttons for client \"%s\" (0x%lx)", c->name, c->window);
 
-    ungrabButton(c->md->dpy, Button1, 0, c->window);
+    ungrabButton(clientGetXDisplay (c), Button1, 0, c->window);
 }
 
 void
