@@ -113,13 +113,13 @@ clientSetNetState (Client * c)
         TRACE ("clientSetNetState : hidden");
         data[i++] = net_wm_state_hidden;
     }
-    XChangeProperty (c->md->dpy, c->window, net_wm_state, XA_ATOM, 32,
+    XChangeProperty (clientGetXDisplay (c), c->window, net_wm_state, XA_ATOM, 32,
         PropModeReplace, (unsigned char *) data, i);
     /*
        We also set GNOME hint here for consistency and convenience, 
        although the meaning of net_wm_state and win_state aren't the same.
      */
-    setHint (c->md->dpy, c->window, win_state, c->win_state);
+    setHint (clientGetXDisplay (c), c->window, win_state, c->win_state);
 }
 
 void
@@ -132,7 +132,7 @@ clientGetNetState (Client * c)
     TRACE ("entering clientGetNetState");
     TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
 
-    if (getAtomList (c->md->dpy, c->window, net_wm_state, &atoms, &n_atoms))
+    if (getAtomList (clientGetXDisplay (c), c->window, net_wm_state, &atoms, &n_atoms))
     {
         int i;
         TRACE ("clientGetNetState: %i atoms detected", n_atoms);
@@ -377,6 +377,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
             clientSetNetState (c);
             clientWindowType (c);
         }
+        frameDraw (c, TRUE, FALSE);
     }
 
     if ((first == net_wm_state_fullscreen)
@@ -388,17 +389,19 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
                 && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
             {
                 FLAG_SET (c->flags, CLIENT_FLAG_FULLSCREEN);
+                clientUpdateFullscreenState (c);
             }
             else if ((action == NET_WM_STATE_REMOVE)
                 && FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
             {
                 FLAG_UNSET (c->flags, CLIENT_FLAG_FULLSCREEN);
+                clientUpdateFullscreenState (c);
             }
             else if (action == NET_WM_STATE_TOGGLE)
             {
                 FLAG_TOGGLE (c->flags, CLIENT_FLAG_FULLSCREEN);
+                clientUpdateFullscreenState (c);
             }
-            clientUpdateFullscreenState (c);
         }
     }
 
@@ -410,17 +413,19 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
                 && !FLAG_TEST (c->flags, CLIENT_FLAG_ABOVE))
             {
                 FLAG_SET (c->flags, CLIENT_FLAG_ABOVE);
+                clientUpdateAboveState (c);
             }
             else if ((action == NET_WM_STATE_REMOVE)
                 && FLAG_TEST (c->flags, CLIENT_FLAG_ABOVE))
             {
                 FLAG_UNSET (c->flags, CLIENT_FLAG_ABOVE);
+                clientUpdateAboveState (c);
             }
             else if (action == NET_WM_STATE_TOGGLE)
             {
                 FLAG_TOGGLE (c->flags, CLIENT_FLAG_ABOVE);
+                clientUpdateAboveState (c);
             }
-            clientUpdateAboveState (c);
         }
     }
 
@@ -432,17 +437,19 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
                 && !FLAG_TEST (c->flags, CLIENT_FLAG_BELOW))
             {
                 FLAG_SET (c->flags, CLIENT_FLAG_BELOW);
+                clientUpdateBelowState (c);
             }
             else if ((action == NET_WM_STATE_REMOVE)
                 && FLAG_TEST (c->flags, CLIENT_FLAG_BELOW))
             {
                 FLAG_UNSET (c->flags, CLIENT_FLAG_BELOW);
+                clientUpdateBelowState (c);
             }
             else if (action == NET_WM_STATE_TOGGLE)
             {
                 FLAG_TOGGLE (c->flags, CLIENT_FLAG_BELOW);
+                clientUpdateBelowState (c);
             }
-            clientUpdateBelowState (c);
         }
     }
 
@@ -510,8 +517,8 @@ clientUpdateFullscreenState (Client * c)
         cx = frameX (c) + (frameWidth (c) / 2);
         cy = frameY (c) + (frameHeight (c) / 2);
 
-        monitor_nbr = gdk_screen_get_monitor_at_point (c->md->gscr, cx, cy);
-        gdk_screen_get_monitor_geometry (c->md->gscr, monitor_nbr, &rect);
+        monitor_nbr = gdk_screen_get_monitor_at_point (c->screen_info->gscr, cx, cy);
+        gdk_screen_get_monitor_geometry (c->screen_info->gscr, monitor_nbr, &rect);
 
         c->fullscreen_old_x = c->x;
         c->fullscreen_old_y = c->y;
@@ -542,10 +549,10 @@ clientUpdateFullscreenState (Client * c)
            grab focus in focus follow mouse mode. Grab the pointer to
            avoid these effects
          */
-        XGrabPointer (c->md->dpy, c->md->gnome_win, FALSE, EnterWindowMask, GrabModeAsync,
+        XGrabPointer (clientGetXDisplay (c), c->screen_info->gnome_win, FALSE, EnterWindowMask, GrabModeAsync,
                            GrabModeAsync, None, None, GDK_CURRENT_TIME);
         clientConfigure (c, &wc, CWX | CWY | CWWidth | CWHeight, NO_CFG_FLAG);
-        XUngrabPointer (c->md->dpy, GDK_CURRENT_TIME);
+        XUngrabPointer (clientGetXDisplay (c), GDK_CURRENT_TIME);
     }
     else
     {
@@ -571,7 +578,7 @@ clientGetNetWmType (Client * c)
     n_atoms = 0;
     atoms = NULL;
 
-    if (!getAtomList (c->md->dpy, c->window, net_wm_window_type, &atoms, &n_atoms))
+    if (!getAtomList (clientGetXDisplay (c), c->window, net_wm_window_type, &atoms, &n_atoms))
     {
         switch (c->win_layer)
         {
@@ -644,9 +651,9 @@ clientGetInitialNetWmDesktop (Client * c)
                 CLIENT_FLAG_SESSION_MANAGED | CLIENT_FLAG_WORKSPACE_SET))
         {
             FLAG_SET (c->flags, CLIENT_FLAG_WORKSPACE_SET);
-            c->win_workspace = c->md->current_ws;
+            c->win_workspace = c->screen_info->current_ws;
         }
-        if (getHint (c->md->dpy, c->window, net_wm_desktop, &val))
+        if (getHint (clientGetXDisplay (c), c->window, net_wm_desktop, &val))
         {
             TRACE ("atom net_wm_desktop detected");
             if (val == (int) ALL_WORKSPACES)
@@ -660,7 +667,7 @@ clientGetInitialNetWmDesktop (Client * c)
                     FLAG_SET (c->flags, CLIENT_FLAG_STICKY);
                     c->win_state |= WIN_STATE_STICKY;
                 }
-                c->win_workspace = c->md->current_ws;
+                c->win_workspace = c->screen_info->current_ws;
             }
             else
             {
@@ -671,7 +678,7 @@ clientGetInitialNetWmDesktop (Client * c)
             }
             FLAG_SET (c->flags, CLIENT_FLAG_WORKSPACE_SET);
         }
-        else if (getHint (c->md->dpy, c->window, win_workspace, &val))
+        else if (getHint (clientGetXDisplay (c), c->window, win_workspace, &val))
         {
             TRACE ("atom win_workspace specifies window \"%s\" is on desk %i",
                 c->name, (int) val);
@@ -681,28 +688,28 @@ clientGetInitialNetWmDesktop (Client * c)
     }
     TRACE ("initial desktop for window \"%s\" is %i", c->name,
         c->win_workspace);
-    if (c->win_workspace > params.workspace_count - 1)
+    if (c->win_workspace > c->screen_info->workspace_count - 1)
     {
         TRACE ("value off limits, using %i instead",
-            params.workspace_count - 1);
-        c->win_workspace = params.workspace_count - 1;
+            c->screen_info->workspace_count - 1);
+        c->win_workspace = c->screen_info->workspace_count - 1;
         FLAG_SET (c->flags, CLIENT_FLAG_WORKSPACE_SET);
     }
     TRACE ("initial desktop for window \"%s\" is %i", c->name,
         c->win_workspace);
-    setHint (c->md->dpy, c->window, win_workspace, c->win_workspace);
+    setHint (clientGetXDisplay (c), c->window, win_workspace, c->win_workspace);
     if (FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
     {
-        setHint (c->md->dpy, c->window, net_wm_desktop, (unsigned long) ALL_WORKSPACES);
+        setHint (clientGetXDisplay (c), c->window, net_wm_desktop, (unsigned long) ALL_WORKSPACES);
     }
     else
     {
-        setHint (c->md->dpy, c->window, net_wm_desktop, (unsigned long) c->win_workspace);
+        setHint (clientGetXDisplay (c), c->window, net_wm_desktop, (unsigned long) c->win_workspace);
     }
 }
 
 void
-clientSetNetClientList (ScreenData * md, Atom a, GList * list)
+clientSetNetClientList (ScreenInfo * screen_info, Atom a, GList * list)
 {
     Window *listw;
     Window *index_dest;
@@ -714,18 +721,18 @@ clientSetNetClientList (ScreenData * md, Atom a, GList * list)
     size = g_list_length (list);
     if (size < 1)
     {
-        XDeleteProperty (md->dpy, md->xroot, a);
+        XDeleteProperty (myScreenGetXDisplay (screen_info), screen_info->xroot, a);
     }
     else if ((listw = (Window *) malloc ((size + 1) * sizeof (Window))))
     {
-        TRACE ("%i windows in list for %i clients", size, client_count);
+        TRACE ("%i windows in list for %i clients", size, screen_info->client_count);
         for (i = 0, index_dest = listw, index_src = list; i < size;
             i++, index_dest++, index_src = g_list_next (index_src))
         {
             Client *c = (Client *) index_src->data;
             *index_dest = c->window;
         }
-        XChangeProperty (md->dpy, md->xroot, a, XA_WINDOW, 32, PropModeReplace,
+        XChangeProperty (myScreenGetXDisplay (screen_info), screen_info->xroot, a, XA_WINDOW, 32, PropModeReplace,
             (unsigned char *) listw, size);
         free (listw);
     }
@@ -749,11 +756,14 @@ clientGetNetStruts (Client * c)
     FLAG_UNSET (c->flags, CLIENT_FLAG_HAS_STRUT);
     FLAG_UNSET (c->flags, CLIENT_FLAG_HAS_STRUT_PARTIAL);
 
-    if (getCardinalList (c->md->dpy, c->window, net_wm_strut_partial, &struts, &nitems))
+    if (getCardinalList (clientGetXDisplay (c), c->window, net_wm_strut_partial, &struts, &nitems))
     {
         if (nitems != 12)
         {
-            XFree (struts);
+            if (struts)
+            {
+                XFree (struts);
+            }
             return;
         }
 
@@ -765,13 +775,16 @@ clientGetNetStruts (Client * c)
         }
 
         XFree (struts);
-        workspaceUpdateArea (c->md);
+        workspaceUpdateArea (c->screen_info);
     }
-    else if (getCardinalList (c->md->dpy, c->window, net_wm_strut, &struts, &nitems))
+    else if (getCardinalList (clientGetXDisplay (c), c->window, net_wm_strut, &struts, &nitems))
     {
         if (nitems != 4)
         {
-            XFree (struts);
+            if (struts)
+            {
+                XFree (struts);
+            }
             return;
         }
 
@@ -787,13 +800,13 @@ clientGetNetStruts (Client * c)
         /* Fill(in values as for partial struts */
         c->struts[TOP_START_X] = c->struts[BOTTOM_START_X] = 0;
         c->struts[TOP_END_X] = c->struts[BOTTOM_END_X] = 
-            gdk_screen_get_width (c->md->gscr);
+            gdk_screen_get_width (c->screen_info->gscr);
         c->struts[LEFT_START_Y] = c->struts[RIGHT_START_Y] = 0;
         c->struts[LEFT_END_Y] = c->struts[RIGHT_END_Y] = 
-            gdk_screen_get_height (c->md->gscr);
+            gdk_screen_get_height (c->screen_info->gscr);
         
         XFree (struts);
-        workspaceUpdateArea (c->md);
+        workspaceUpdateArea (c->screen_info);
     }
 }
 
@@ -818,14 +831,14 @@ clientSetNetActions (Client * c)
     {
         atoms[i++] = net_wm_action_shade;
     }
-    XChangeProperty (c->md->dpy, c->window, net_wm_allowed_actions, XA_ATOM, 32,
+    XChangeProperty (clientGetXDisplay (c), c->window, net_wm_allowed_actions, XA_ATOM, 32,
         PropModeReplace, (unsigned char *) atoms, i);
 }
 
 void
 clientWindowType (Client * c)
 {
-    WindowType old_type;
+    netWindowType old_type;
 
     g_return_if_fail (c != NULL);
     TRACE ("entering clientWindowType");
